﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Net;
using System.Net.Http;
using System.Threading;
using Newtonsoft.Json;
using System.Web.Http.Cors;

namespace WebApiSelfHostAuthorization
{
    public class BasicAuthorizationHandler:DelegatingHandler
    {
        //protected  override Task<HttpResponseMessage> SendAsync(HttpRequestMessage request, CancellationToken cancellationToken)
        //{
        //    if (!ValidateRequest(request))
        //    {
        //        //没有通过则创建response,code为401
        //        var response = new HttpResponseMessage(HttpStatusCode.Forbidden);//401未授权
        //        var tsc = new TaskCompletionSource<HttpResponseMessage>();
        //        tsc.SetResult(response);
        //        return tsc.Task;
        //    }
        //    // Call the inner handler.
        //    //var response = await base.SendAsync(request, cancellationToken);
        //    //response.Headers.Add("Tip", "test");//可以为响应消息添加自定义标头
        //    //System.Diagnostics.Debug.WriteLine("Process response");
        //    return base.SendAsync(request, cancellationToken);
        //}
        protected async override Task<HttpResponseMessage> SendAsync(HttpRequestMessage request, CancellationToken cancellationToken)
        {
            //包含自定义header字段的跨域请求，浏览器会先向服务器发送OPTIONS请求，探测该服务器是否允许自定义的跨域字段。如果允许，则继续实际的POST／GET正常请求，否则，返回错误。https://blog.csdn.net/xuedapeng/article/details/79076704
            //必须理解在正常请求前会收到option请求，如果不让其通过message handler，则既不能通过身份验证（没有authorization头），也不能通过跨域访问控制（在本类中即被返回response，根本到达不了路由配置处），因此在前端页面会出现两种错误让人无法理解
            if (request.Method == HttpMethod.Options)
            {
                //如果是前端用于验证跨域允许的option请求，则将其转到路由配置处处理，路由跨域配置处理后前端会发送真正的请求，由下面的身份验证代码处理
                var optRes = await base.SendAsync(request, cancellationToken);
                return optRes;
            }
            if (!ValidateRequest(request))
            {
                //注意：和上面同样道理，假如身份验证失败，该请求没有进入路由跨域配置（真正的请求而非option），直接返回后前端会出现跨域错误。不过问题不大，此时是有403和cors两种错误，前端能据此定位错误。

                //没有通过则创建response,code为401
                var response = new HttpResponseMessage(HttpStatusCode.Forbidden);//用403拒绝访问。如果用401未授权且有页面的话会被重定向到登录页
                var content=new Result{
                    success=false,
                    errs=new []{"服务端拒绝访问：你没有权限"}
                };
                //添加响应头返回给跨域请求
                //response.Headers.Add("Access-Control-Allow-Origin", "*");
                //response.Headers.Add("Cache-Control", "no-cache");
                //response.Headers.Add("Access-Control-Allow-Headers", "Content-Type,Authorization，token");//允许自定义头
                //response.Headers.Add("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS");
              
                response.Content = new StringContent(JsonConvert.SerializeObject(content), Encoding.UTF8, "application/json");//content添加错误信息
                return response;//返回响应
            }
            var res = await base.SendAsync(request, cancellationToken);
            return res;
            //以下为官方文档用法，这种方法会使被拒绝方没有任何提示信息
            //var tsc = new TaskCompletionSource<HttpResponseMessage>();
            //tsc.SetResult(response); // Also sets the task state to "RanToCompletion"
            //return tsc.Task;
        }
        private bool ValidateRequest(HttpRequestMessage message)//验证信息解密并对比
        {
            var authorization = message.Headers.Authorization;
            //如果此header为空或不是basic方式则返回未授权
            if (authorization != null && authorization.Scheme == "Basic" && authorization.Parameter != null)
            {
                string Parameter = DES1.Decrypt(authorization.Parameter);
                //string Parameter = DES.Decrypt(authorization.Parameter);
                //string Parameter = System.Text.Encoding.UTF8.GetString(Convert.FromBase64String(authorization.Parameter));//将base64转为字符串形式
                return (Parameter == "lbh:123456");
                //return (Parameter == "你好");//测试中文

            }
            else
            {
                return false;
            }
        }
    
    }
    public class Result//构建用于返回错误信息的对象
    {
        public bool success { get; set; }
        public string[] errs { get; set; }
    }
}
